import 'dart:io';
import 'dart:typed_data';
import 'package:fis_common/logger/logger.dart';
import 'package:fis_vid/common/vid_download.dart';
import 'package:path_provider/path_provider.dart';

/// Vid原生应用缓存
class VidNativeCache {
  // ignore: constant_identifier_names
  static const C_STORAGE_DIR = 'ExamRecords';

  static VidNativeCache? _ins;

  /// 单例
  static VidNativeCache get ins {
    _ins ??= VidNativeCache();
    return _ins!;
  }

  /// 获取缓存
  Future<Uint8List?> getCache(String url) async {
    try {
      final file = await _getCacheFile(url);
      if (await file.exists()) {
        final buffer = await file.readAsBytes();
        return buffer;
      }
    } catch (e) {
      logger.e('VidNativeCache - get cache fail', e);
    }
    return null;
  }

  Future<bool> saveCache(String url, Uint8List bytes) async {
    try {
      final file = await _getCacheFile(url);
      final newFile = await file.create(recursive: true);
      newFile.writeAsBytes(bytes);
      return true;
    } catch (e) {
      logger.e('VidNativeCache - save cache fail', e);
    }
    return false;
  }

  Future<File> _getCacheFile(String url) async {
    final name = url.split('/').last.split('.').first;
    final fileName = '$name.dat';
    final cachePath = await _getStoragePath(fileName);
    final file = File(cachePath);
    return file;
  }

  Future<String> _getStoragePath(String fileName) async {
    Directory? dir;
    if (Platform.isAndroid) {
      dir = await getExternalStorageDirectory();
    }
    dir ??= await getApplicationDocumentsDirectory();
    return "${dir.path}/$C_STORAGE_DIR/$fileName";
  }
}
