import 'dart:convert';
import 'package:fis_common/logger/logger.dart';
import 'package:fis_vid/data_host/interface/data_host.dart';
import 'package:fis_vid/common/vid_download.dart';
import 'package:fis_vid/common/env.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:vid/us/vid_us_image.dart';
import 'package:vid/us/vid_us_image_data.dart';
import 'package:vid/us/vid_us_probe.dart';

import '../base/data_host.dart';

part 'broswer.dart';
part 'shell.dart';

class VidDataHost extends VidDataHostBase {
  VidDataHost(String url) : super(url);

  VidDataHostInterface? _hostInstance;

  @override
  int get frameCount => _hostInstance!.frameCount;

  @override
  VidUsProbe get probe => _hostInstance!.probe;

  @override
  Future<VidUsImageData?> getData() async => await _hostInstance?.getData();

  @override
  Future<VidUsImage?> getFrame(int index) async {
    if (_hostInstance == null) {
      throw Exception(
          "[VidDataHost] getFrame: must call load first and data is not null.");
    }
    final frame = await _hostInstance!.getFrame(index);
    if (frame != null) {
      frameLoaded.emit(this, frame);
    }
    return frame;
  }

  @override
  Future<VidDataHostLoadInfo?> load() async {
    if (_hostInstance != null) {
      await _hostInstance!.release();
      _hostInstance = null;
    }
    _hostInstance = VidDataHostEnv.isShell
        ? _VidDataHostShell(url)
        : _VidDataHostBroswer(url);
    return await _hostInstance!.load();
  }

  @override
  Future<void> release() async {
    await _hostInstance?.release();
    _hostInstance = null;
  }
}
